@@ -1,4 +1,6 @@
 /*
+ *  Copyright (C) Stephan Arts 2006-2008 <stephan@xfce.org>
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -24,7 +26,7 @@
 #include <dbus/dbus-glib.h>
 
 #include "navigator.h"
-#include "thumbnail_viewer.h"
+#include "thumbnail_bar.h"
 #include "picture_viewer.h"
 #include "main_window.h"
 
@@ -298,7 +300,7 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
 
     window->priv->navigator = rstto_navigator_new();
-    window->priv->thumbnail_viewer = rstto_thumbnail_viewer_new(window->priv->navigator);
+    window->priv->thumbnail_viewer = rstto_thumbnail_bar_new(window->priv->navigator);
     window->priv->picture_viewer = rstto_picture_viewer_new(window->priv->navigator);
 
     window->priv->manager = gtk_recent_manager_get_default();
@@ -862,7 +864,7 @@ rstto_main_window_set_thumbnail_viewer_orientation(RsttoMainWindow *window, GtkO
     gtk_container_remove(GTK_CONTAINER(window->priv->containers.paned), window->priv->containers.s_window);
     gtk_widget_destroy(window->priv->containers.paned);
 
-    rstto_thumbnail_viewer_set_orientation(RSTTO_THUMBNAIL_VIEWER(window->priv->thumbnail_viewer), orientation);
+    rstto_thumbnail_bar_set_orientation(RSTTO_THUMBNAIL_BAR(window->priv->thumbnail_viewer), orientation);
 
     switch (orientation)
     {
@@ -889,7 +891,7 @@ rstto_main_window_set_show_thumbnail_viewer(RsttoMainWindow *window, gboolean vi
     window->priv->settings.thumbnail_viewer_visibility = visibility;
     if (visibility == TRUE)
     {
-        switch(rstto_thumbnail_viewer_get_orientation(RSTTO_THUMBNAIL_VIEWER(window->priv->thumbnail_viewer)))
+        switch(rstto_thumbnail_bar_get_orientation(RSTTO_THUMBNAIL_BAR(window->priv->thumbnail_viewer)))
         {
             case GTK_ORIENTATION_HORIZONTAL:
                 gtk_check_menu_item_set_active(
@@ -974,6 +976,7 @@ void
 rstto_main_window_set_max_cache_size (RsttoMainWindow *window, gint max_cache_size)
 {
     window->priv->settings.max_cache_size = max_cache_size;
+    rstto_navigator_set_max_history_size(window->priv->navigator, max_cache_size * 1000000);
 }
 
 /* CALLBACK FUNCTIONS */
@@ -1158,6 +1161,21 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     GtkWidget *slideshow_main_lbl;
     GtkWidget *display_main_vbox;
     GtkWidget *display_main_lbl;
+    GtkWidget *bg_color_vbox;
+    GtkWidget *bg_color_hbox;
+    GtkWidget *bg_color_frame;
+    GtkWidget *bg_color_button;
+    GtkWidget *bg_color_override_check;
+    GtkWidget *cache_vbox;
+    GtkWidget *cache_frame;
+    GtkWidget *cache_hbox;
+    GtkObject *cache_adjustment;
+    GtkWidget *cache_spin_button;
+    GtkWidget *cache_label;
+    GtkWidget *cache_mb_label;
+
+    cache_adjustment = gtk_adjustment_new(rstto_main_window_get_max_cache_size(window), 0, 9999, 1, 100, 100);
+
     GtkWidget *dialog = xfce_titled_dialog_new_with_buttons(_("Image viewer Preferences"),
                                                     GTK_WINDOW(window),
                                                     GTK_DIALOG_NO_SEPARATOR,
@@ -1182,15 +1200,16 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     display_main_lbl = gtk_label_new(_("Display"));
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), display_main_vbox, display_main_lbl);
 
-    GtkWidget *bg_color_vbox = gtk_vbox_new(FALSE, 0);
-    GtkWidget *bg_color_frame = xfce_create_framebox_with_content (_("Background Color"), bg_color_vbox);
+    bg_color_vbox = gtk_vbox_new(FALSE, 0);
+    bg_color_frame = xfce_create_framebox_with_content (_("Background Color"), bg_color_vbox);
 
-    GtkWidget *bg_color_override_check = gtk_check_button_new_with_mnemonic(_("_Override Background Color:"));
-    GtkWidget *bg_hbox = gtk_hbox_new(FALSE, 4);
-    GtkWidget *bg_color_button = gtk_color_button_new();
-    gtk_box_pack_start(GTK_BOX(bg_hbox), bg_color_override_check, FALSE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(bg_hbox), bg_color_button, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(bg_color_vbox), bg_hbox, FALSE, FALSE, 0);
+    bg_color_override_check = gtk_check_button_new_with_mnemonic(_("_Override Background Color:"));
+    bg_color_hbox = gtk_hbox_new(FALSE, 4);
+    bg_color_button = gtk_color_button_new();
+
+    gtk_box_pack_start(GTK_BOX(bg_color_hbox), bg_color_override_check, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(bg_color_hbox), bg_color_button, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(bg_color_vbox), bg_color_hbox, FALSE, FALSE, 0);
 
     g_signal_connect(G_OBJECT(bg_color_override_check), "toggled", (GCallback)cb_rstto_bg_color_override_check_toggled, bg_color_button);
 
@@ -1206,9 +1225,24 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
         gtk_widget_set_sensitive(bg_color_button, FALSE);
     }
 
+    cache_vbox = gtk_vbox_new(FALSE, 0);
+    cache_frame = xfce_create_framebox_with_content (_("Image Cache"), cache_vbox);
+    cache_hbox = gtk_hbox_new(FALSE, 4);
+    cache_spin_button = gtk_spin_button_new(GTK_ADJUSTMENT(cache_adjustment), 1.0, 0);
+    cache_label = gtk_label_new(_("Cache size:"));
+    cache_mb_label = gtk_label_new(_("MB"));
+
+    gtk_box_pack_start(GTK_BOX(cache_hbox), cache_label, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(cache_hbox), cache_spin_button, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(cache_hbox), cache_mb_label, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(cache_vbox), cache_hbox, FALSE, FALSE, 0);
+
+
     gtk_container_set_border_width (GTK_CONTAINER (bg_color_frame), 8);
+    gtk_container_set_border_width (GTK_CONTAINER (cache_frame), 8);
 
     gtk_box_pack_start(GTK_BOX(display_main_vbox), bg_color_frame, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(display_main_vbox), cache_frame, FALSE, TRUE, 0);
 
     GtkWidget *slideshow_vbox = gtk_vbox_new(FALSE, 0);
     GtkWidget *slideshow_frame = xfce_create_framebox_with_content (_("Timeout"), slideshow_vbox);
@@ -1266,6 +1300,7 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
                 rstto_main_window_set_pv_bg_color(window, NULL);
             }
             rstto_picture_viewer_redraw(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer));
+            rstto_main_window_set_max_cache_size(window, GTK_ADJUSTMENT(cache_adjustment)->value);
         default:
             break;
     }
@@ -1335,7 +1370,7 @@ cb_rstto_main_window_open_file(GtkWidget *widget, RsttoMainWindow *window)
             if(!strcmp(file_media, "image"))
             {
                 RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
-                rstto_navigator_add (window->priv->navigator, entry);
+                rstto_navigator_add (window->priv->navigator, entry, TRUE);
                 gchar *uri = thunar_vfs_path_dup_uri(info->path);
                 gtk_recent_manager_add_item(window->priv->manager, uri);
                 g_free(uri);
@@ -1375,6 +1410,14 @@ cb_rstto_main_window_open_folder(GtkWidget *widget, RsttoMainWindow *window)
         GDir *dir = g_dir_open(dir_name, 0, NULL);
         if (dir)
         {
+            ThunarVfsPath *dir_path = thunar_vfs_path_new(dir_name, NULL);
+            if (dir_path)
+            {
+                rstto_navigator_set_monitor_handle_for_dir(window->priv->navigator, dir_path);
+                thunar_vfs_path_unref(dir_path);
+                dir_path = NULL;
+            }
+
             const gchar *filename = g_dir_read_name(dir);
             while (filename)
             {
@@ -1387,7 +1430,7 @@ cb_rstto_main_window_open_folder(GtkWidget *widget, RsttoMainWindow *window)
                     if(!strcmp(file_media, "image"))
                     {
                         RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
-                        rstto_navigator_add (window->priv->navigator, entry);
+                        rstto_navigator_add (window->priv->navigator, entry, FALSE);
                     }
                     g_free(file_media);
                     thunar_vfs_path_unref(path);
@@ -1418,13 +1461,15 @@ cb_rstto_main_window_open_recent(GtkRecentChooser *chooser, RsttoMainWindow *win
             if(strcmp(thunar_vfs_mime_info_get_name(info->mime_info), "inode/directory"))
             {
                 RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
-                rstto_navigator_add (window->priv->navigator, entry);
+                rstto_navigator_add (window->priv->navigator, entry, TRUE);
             }
             else
             {
                 rstto_navigator_clear(window->priv->navigator);
                 gchar *dir_path = thunar_vfs_path_dup_string(info->path);
                 GDir *dir = g_dir_open(dir_path, 0, NULL);
+                rstto_navigator_set_monitor_handle_for_dir(window->priv->navigator, info->path);
+
                 const gchar *filename = g_dir_read_name(dir);
                 while (filename)
                 {
@@ -1437,7 +1482,7 @@ cb_rstto_main_window_open_recent(GtkRecentChooser *chooser, RsttoMainWindow *win
                         if(!strcmp(file_media, "image"))
                         {
                             RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, file_info);
-                            rstto_navigator_add (window->priv->navigator, entry);
+                            rstto_navigator_add (window->priv->navigator, entry, FALSE);
                         }
                         g_free(file_media);
                         thunar_vfs_path_unref(file_path);
@@ -1743,6 +1788,7 @@ cb_rstto_main_window_last(GtkWidget *widget, RsttoMainWindow *window)
 static void
 cb_rstto_main_window_zoom_in(GtkWidget *widget, RsttoMainWindow *window)
 {
+    rstto_picture_viewer_set_zoom_mode(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), RSTTO_ZOOM_MODE_CUSTOM);
     gdouble scale = rstto_picture_viewer_get_scale(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer));
     rstto_picture_viewer_set_scale(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), scale * window->priv->zoom_factor);
 }
@@ -1750,6 +1796,7 @@ cb_rstto_main_window_zoom_in(GtkWidget *widget, RsttoMainWindow *window)
 static void
 cb_rstto_main_window_zoom_out(GtkWidget *widget, RsttoMainWindow *window)
 {
+    rstto_picture_viewer_set_zoom_mode(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), RSTTO_ZOOM_MODE_CUSTOM);
     gdouble scale = rstto_picture_viewer_get_scale(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer));
     rstto_picture_viewer_set_scale(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), scale / window->priv->zoom_factor);
 }
@@ -1757,12 +1804,14 @@ cb_rstto_main_window_zoom_out(GtkWidget *widget, RsttoMainWindow *window)
 static void
 cb_rstto_main_window_zoom_100(GtkWidget *widget, RsttoMainWindow *window)
 {
+    rstto_picture_viewer_set_zoom_mode(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), RSTTO_ZOOM_MODE_100);
     rstto_picture_viewer_set_scale(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), 1);
 }
 
 static void
 cb_rstto_main_window_zoom_fit(GtkWidget *widget, RsttoMainWindow *window)
 {
+    rstto_picture_viewer_set_zoom_mode(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), RSTTO_ZOOM_MODE_FIT);
     rstto_picture_viewer_fit_scale(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer));
 }
 
@@ -1849,7 +1898,6 @@ rstto_main_window_set_pv_bg_color (RsttoMainWindow *window, const GdkColor *colo
 const GdkColor *
 rstto_main_window_get_pv_bg_color (RsttoMainWindow *window)
 {
-    /*return rstto_picture_viewer_get_bg_color(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer));*/
     return window->priv->settings.bg_color;
 }
 
