@@ -56,11 +56,21 @@ cb_rstto_navigator_entry_fs_event (ThunarVfsMonitor *,
                                    ThunarVfsPath *,
                                    ThunarVfsPath *,
                                    RsttoNavigatorEntry *);
+static void
+cb_rstto_navigator_fs_event (ThunarVfsMonitor *monitor,
+                             ThunarVfsMonitorHandle *handl,
+                             ThunarVfsMonitorEvent event,
+                             ThunarVfsPath *handle_path,
+                             ThunarVfsPath *event_path,
+                             RsttoNavigator *nav);
 
+static gint
+cb_rstto_navigator_entry_path_compare_func(RsttoNavigatorEntry *entry, ThunarVfsPath *path);
 
 enum
 {
     RSTTO_NAVIGATOR_SIGNAL_ENTRY_MODIFIED = 0,
+    RSTTO_NAVIGATOR_SIGNAL_ENTRY_REMOVED,
     RSTTO_NAVIGATOR_SIGNAL_NEW_ENTRY,
     RSTTO_NAVIGATOR_SIGNAL_ITER_CHANGED,
     RSTTO_NAVIGATOR_SIGNAL_REORDERED,
@@ -86,7 +96,6 @@ struct _RsttoNavigatorEntry
 
     RsttoNavigator      *navigator;
 
-
     gdouble              scale;
     gboolean             fit_to_screen;
     GdkPixbufRotation    rotation;
@@ -131,9 +140,11 @@ rstto_navigator_init(RsttoNavigator *navigator)
     navigator->old_position = -1;
     navigator->timeout = 5000;
     navigator->monitor = thunar_vfs_monitor_get_default();
-    navigator->max_history = 0;
     navigator->preload = FALSE;
 
+    /* Max history size (in bytes) */
+    navigator->max_history = 128000000;
+
     navigator->factory = thunar_vfs_thumb_factory_new(THUNAR_VFS_THUMB_SIZE_NORMAL);
 }
 
@@ -167,6 +178,17 @@ rstto_navigator_class_init(RsttoNavigatorClass *nav_class)
             1,
             G_TYPE_POINTER,
             NULL);
+    rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ENTRY_REMOVED] = g_signal_new("entry-removed",
+            G_TYPE_FROM_CLASS(nav_class),
+            G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+            0,
+            NULL,
+            NULL,
+            g_cclosure_marshal_VOID__POINTER,
+            G_TYPE_NONE,
+            1,
+            G_TYPE_POINTER,
+            NULL);
     rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ITER_CHANGED] = g_signal_new("iter-changed",
             G_TYPE_FROM_CLASS(nav_class),
             G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
@@ -268,53 +290,59 @@ rstto_navigator_guard_history(RsttoNavigator *navigator, RsttoNavigatorEntry *en
 
     /* add image to the cache-history */
     navigator->history = g_list_prepend(navigator->history, entry);
+    
+    GList *iter = NULL;
+    guint64 size = 0;
 
-    if (g_list_length(navigator->history) > navigator->max_history)
+    for (iter = navigator->history; iter != NULL; iter = g_list_next(iter))
     {
-        GList *last_entry = g_list_last(navigator->history);
-        RsttoNavigatorEntry *nav_entry = last_entry->data;
+        RsttoNavigatorEntry *nav_entry = iter->data;
 
         if (nav_entry)
         {
+            size += rstto_navigator_entry_get_size(nav_entry);
 
-            if(nav_entry->thumb)
+            if (size > navigator->max_history)
             {
-                gdk_pixbuf_unref(nav_entry->thumb);
-                nav_entry->thumb = NULL;
-            }
+                if(nav_entry->thumb)
+                {
+                    gdk_pixbuf_unref(nav_entry->thumb);
+                    nav_entry->thumb = NULL;
+                }
 
-            if(nav_entry->io_channel)
-            {
-                g_source_remove(nav_entry->io_source_id);
-                g_io_channel_unref(nav_entry->io_channel);
-                nav_entry->io_channel = NULL;
-                nav_entry->io_source_id = 0;
-            }
+                if(nav_entry->io_channel)
+                {
+                    g_source_remove(nav_entry->io_source_id);
+                    g_io_channel_unref(nav_entry->io_channel);
+                    nav_entry->io_channel = NULL;
+                    nav_entry->io_source_id = 0;
+                }
 
-            if (entry->timeout_id)
-            {
-                g_source_remove(entry->timeout_id);
-                entry->timeout_id = 0;
-            }
+                if (entry->timeout_id)
+                {
+                    g_source_remove(entry->timeout_id);
+                    entry->timeout_id = 0;
+                }
 
-            if(nav_entry->loader)
-            {
-                g_signal_handlers_disconnect_by_func(nav_entry->loader , cb_rstto_navigator_entry_area_prepared, nav_entry);
-                gdk_pixbuf_loader_close(nav_entry->loader, NULL);
-            }
-            if(nav_entry->animation)
-            {
-                g_object_unref(nav_entry->animation);
-                nav_entry->animation = NULL;
-            }
-            if(nav_entry->src_pixbuf)
-            {
-                gdk_pixbuf_unref(nav_entry->src_pixbuf);
-                nav_entry->src_pixbuf = NULL;
+                if(nav_entry->loader)
+                {
+                    g_signal_handlers_disconnect_by_func(nav_entry->loader , cb_rstto_navigator_entry_area_prepared, nav_entry);
+                    gdk_pixbuf_loader_close(nav_entry->loader, NULL);
+                }
+                if(nav_entry->animation)
+                {
+                    g_object_unref(nav_entry->animation);
+                    nav_entry->animation = NULL;
+                }
+                if(nav_entry->src_pixbuf)
+                {
+                    gdk_pixbuf_unref(nav_entry->src_pixbuf);
+                    nav_entry->src_pixbuf = NULL;
+                }
+                iter = g_list_previous(iter);
+                navigator->history = g_list_remove(navigator->history, nav_entry);
             }
         }
-
-        navigator->history = g_list_remove(navigator->history, nav_entry);
     }
 }
 
@@ -479,7 +507,7 @@ rstto_navigator_get_nth_file (RsttoNavigator *navigator, gint n)
 }
 
 gint
-rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
+rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry, gboolean with_monitor)
 {
     g_return_val_if_fail(navigator == entry->navigator, -1);
 
@@ -495,7 +523,8 @@ rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
                       NULL);
     }
 
-    entry->monitor_handle = thunar_vfs_monitor_add_file(navigator->monitor, entry->info->path, (ThunarVfsMonitorCallback)cb_rstto_navigator_entry_fs_event, entry);
+    if (with_monitor == TRUE)
+        entry->monitor_handle = thunar_vfs_monitor_add_file(navigator->monitor, entry->info->path, (ThunarVfsMonitorCallback)cb_rstto_navigator_entry_fs_event, entry);
 
     g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_NEW_ENTRY], 0, g_list_index(navigator->file_list, entry), entry, NULL);
     return g_list_index(navigator->file_list, entry);
@@ -510,16 +539,22 @@ rstto_navigator_remove (RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
         {
             navigator->old_position = rstto_navigator_get_position(navigator);
             navigator->file_iter = g_list_next(navigator->file_iter);
+
+            navigator->file_list = g_list_remove(navigator->file_list, entry);
+
             if(!navigator->file_iter)
                 navigator->file_iter = g_list_first(navigator->file_list);
 
-            navigator->file_list = g_list_remove(navigator->file_list, entry);
+            /* Ehm... an item can exist several times inside the history? */
+            if (navigator->history)
+                navigator->history = g_list_remove_all(navigator->history, entry);
+
+            g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ENTRY_REMOVED], 0, entry, NULL);
             if(g_list_length(navigator->file_list) == 0)
             {
                 navigator->file_iter = NULL;
                 navigator->file_list = NULL;
             }
-            g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_REORDERED], 0, NULL);
             if(navigator->file_iter)
             {
                 g_signal_emit(G_OBJECT(navigator),
@@ -556,7 +591,6 @@ rstto_navigator_remove (RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
         navigator->file_iter = NULL;
         navigator->file_list = NULL;
     }
-    g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_REORDERED], 0, NULL);
 }
 
 void
@@ -564,10 +598,12 @@ rstto_navigator_clear (RsttoNavigator *navigator)
 {
     if(navigator->file_list)
     {
+        g_list_free(navigator->history);
         g_list_foreach(navigator->file_list, (GFunc)rstto_navigator_entry_free, NULL);
         navigator->file_list = NULL;
         navigator->file_iter = NULL;
         navigator->old_position = -1;
+        navigator->history = NULL;
     }
     g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ITER_CHANGED], 0, -1, NULL, NULL);
     g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_REORDERED], 0, NULL);
@@ -575,7 +611,7 @@ rstto_navigator_clear (RsttoNavigator *navigator)
 }
 
 void
-rstto_navigator_set_file (RsttoNavigator *navigator, gint n)
+rstto_navigator_set_file_nr (RsttoNavigator *navigator, gint n)
 {
     if(navigator->file_iter)
     {
@@ -782,7 +818,7 @@ rstto_navigator_entry_free(RsttoNavigatorEntry *nav_entry)
 }
 
 GdkPixbuf *
-rstto_navigator_get_entry_thumb(RsttoNavigator *navigator, RsttoNavigatorEntry *entry, gint size)
+rstto_navigator_entry_get_thumb(RsttoNavigatorEntry *entry, gint size)
 {
     if(entry->thumb)    
     {
@@ -793,13 +829,13 @@ rstto_navigator_get_entry_thumb(RsttoNavigator *navigator, RsttoNavigatorEntry *
     else
     {
         ThunarVfsInfo *info = rstto_navigator_entry_get_info(entry);
-        gchar *thumbnail = thunar_vfs_thumb_factory_lookup_thumbnail(navigator->factory, info);
+        gchar *thumbnail = thunar_vfs_thumb_factory_lookup_thumbnail(entry->navigator->factory, info);
         if (thumbnail == NULL)
         {
-            GdkPixbuf *pixbuf = thunar_vfs_thumb_factory_generate_thumbnail(navigator->factory, info);
+            GdkPixbuf *pixbuf = thunar_vfs_thumb_factory_generate_thumbnail(entry->navigator->factory, info);
             if (pixbuf != NULL)
             {
-                if (!thunar_vfs_thumb_factory_store_thumbnail(navigator->factory, pixbuf, info, NULL))
+                if (!thunar_vfs_thumb_factory_store_thumbnail(entry->navigator->factory, pixbuf, info, NULL))
                 {
                     g_critical("Storing thumbnail failed");
                 }
@@ -809,11 +845,19 @@ rstto_navigator_get_entry_thumb(RsttoNavigator *navigator, RsttoNavigatorEntry *
 
                 if (width > height)
                 {
-                    entry->thumb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, gdk_pixbuf_get_bits_per_sample(pixbuf), size, height*size/width);
+                    entry->thumb = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
+                                                  gdk_pixbuf_get_has_alpha(pixbuf),
+                                                  gdk_pixbuf_get_bits_per_sample(pixbuf),
+                                                  size,
+                                                  height*size/width);
                 }
                 else
                 {
-                    entry->thumb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, gdk_pixbuf_get_bits_per_sample(pixbuf), width*size/height, size);
+                    entry->thumb = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
+                                                  gdk_pixbuf_get_has_alpha(pixbuf),
+                                                  gdk_pixbuf_get_bits_per_sample(pixbuf),
+                                                  width*size/height,
+                                                  size);
                 }
                 gdk_pixbuf_scale(pixbuf, entry->thumb,
                                  0, 0, 
@@ -912,6 +956,21 @@ rstto_navigator_entry_get_pixbuf (RsttoNavigatorEntry *entry)
     return entry->src_pixbuf;
 }
 
+guint64
+rstto_navigator_entry_get_size (RsttoNavigatorEntry *entry)
+{
+    if (entry->src_pixbuf)
+    {
+        gint width = gdk_pixbuf_get_width(entry->src_pixbuf);
+        gint height = gdk_pixbuf_get_height(entry->src_pixbuf);
+
+        gint n_channels = gdk_pixbuf_get_n_channels(entry->src_pixbuf);
+
+        return (guint64) width * height * n_channels;
+    }
+    return 0;
+}
+
 gboolean
 rstto_navigator_entry_load_image (RsttoNavigatorEntry *entry, gboolean empty_cache)
 {
@@ -1019,6 +1078,13 @@ cb_rstto_navigator_entry_area_prepared (GdkPixbufLoader *loader, RsttoNavigatorE
 
     if (time != -1)
     {
+        /* fix borked stuff */
+        if (time == 0)
+        {
+            g_warning("timeout == 0: defaulting to 40ms");
+            time = 40;
+        }
+
         entry->timeout_id = g_timeout_add(time, (GSourceFunc)cb_rstto_navigator_entry_update_image, entry);
     }   
     else
@@ -1118,6 +1184,11 @@ cb_rstto_navigator_entry_update_image (RsttoNavigatorEntry *entry)
         gint time = gdk_pixbuf_animation_iter_get_delay_time(entry->iter);
         if (time != -1)
         {
+            if (time == 0)
+            {
+                g_warning("timeout == 0: defaulting to 40ms");
+                time = 40;
+            }
             entry->timeout_id = g_timeout_add(time, (GSourceFunc)cb_rstto_navigator_entry_update_image, entry);
         }
         g_signal_emit(G_OBJECT(entry->navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ENTRY_MODIFIED], 0, entry, NULL);
@@ -1151,8 +1222,131 @@ cb_rstto_navigator_entry_fs_event (ThunarVfsMonitor *monitor,
     }
 }
 
-gint
-rstto_navigator_get_cache_max_images (RsttoNavigator *navigator)
+static void
+cb_rstto_navigator_fs_event (ThunarVfsMonitor *monitor,
+                             ThunarVfsMonitorHandle *handl,
+                             ThunarVfsMonitorEvent event,
+                             ThunarVfsPath *handle_path,
+                             ThunarVfsPath *event_path,
+                             RsttoNavigator *nav)
+{
+    RsttoNavigatorEntry *entry = NULL;
+    GList *iter = g_list_find_custom(nav->file_list, event_path, (GCompareFunc)cb_rstto_navigator_entry_path_compare_func);
+    if (iter != NULL)
+        entry = iter->data;
+
+    switch (event)
+    {
+        case THUNAR_VFS_MONITOR_EVENT_CHANGED:
+            if(entry)
+            {
+                rstto_navigator_entry_load_image (entry, TRUE);
+            }
+            break;
+        case THUNAR_VFS_MONITOR_EVENT_CREATED:
+            if (entry)
+            {
+                g_critical("File created... yet it is already here");
+                rstto_navigator_remove(entry->navigator, entry);
+                rstto_navigator_entry_free(entry);
+            }
+
+            ThunarVfsInfo *info = thunar_vfs_info_new_for_path(event_path, NULL);
+            if (info)
+            {
+                gchar *file_media = thunar_vfs_mime_info_get_media(info->mime_info);
+                if(!strcmp(file_media, "image"))
+                {
+                    entry = rstto_navigator_entry_new(nav, info);
+                    rstto_navigator_add (nav, entry, FALSE);
+                }
+                g_free(file_media);
+            }
+            break;
+        case THUNAR_VFS_MONITOR_EVENT_DELETED:
+            if(entry)
+            {
+                rstto_navigator_remove(entry->navigator, entry);
+                rstto_navigator_entry_free(entry);
+            }
+            break;
+        default:
+            break;
+    }
+}
+
+void
+rstto_navigator_entry_select (RsttoNavigatorEntry *entry)
+{
+    RsttoNavigator *navigator = entry->navigator;
+    GList *iter = g_list_find (navigator->file_list, entry);
+    if (iter)
+    {
+        if(navigator->file_iter)
+        {
+            rstto_navigator_guard_history(navigator, navigator->file_iter->data);
+            navigator->old_position = rstto_navigator_get_position(navigator);
+        }
+
+        navigator->file_iter = iter;
+
+        g_signal_emit(G_OBJECT(navigator),
+                      rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ITER_CHANGED],
+                      0,
+                      g_list_position(navigator->file_list, navigator->file_iter),
+                      navigator->file_iter->data,
+                      NULL);
+    }
+
+}
+
+gdouble
+rstto_navigator_get_max_history_size (RsttoNavigator *navigator)
 {
     return navigator->max_history;
 }
+
+
+
+gint
+rstto_navigator_entry_get_position(RsttoNavigatorEntry *entry)
+{
+    RsttoNavigator *navigator = entry->navigator;
+    GList *iter = g_list_find (navigator->file_list, entry);
+    if (iter)
+    {
+        return g_list_position(navigator->file_list, iter);
+    }
+    return -1;
+}
+
+void
+rstto_navigator_set_max_history_size(RsttoNavigator *nav, gdouble size)
+{
+    nav->max_history = size;
+}
+
+void
+rstto_navigator_set_monitor_handle_for_dir(RsttoNavigator *nav, ThunarVfsPath *dir_path)
+{
+    if (nav->monitor_handle)
+    {
+        thunar_vfs_monitor_remove(nav->monitor, nav->monitor_handle);
+        nav->monitor_handle = NULL;
+    }
+    
+    if (dir_path)
+    {
+        nav->monitor_handle = thunar_vfs_monitor_add_directory(nav->monitor, dir_path, (ThunarVfsMonitorCallback)cb_rstto_navigator_fs_event, nav);
+    }
+}
+
+static gint
+cb_rstto_navigator_entry_path_compare_func(RsttoNavigatorEntry *entry, ThunarVfsPath *path)
+{
+    if (thunar_vfs_path_equal(entry->info->path, path) == TRUE)
+    {
+        return 0;
+    }
+    return 1;
+}
