@@ -47,6 +47,8 @@ struct _RsttoNavigator
 {
     GObject                parent;
     ThunarVfsMonitor      *monitor;
+    ThunarVfsMonitorHandle *monitor_handle;
+
     ThunarVfsThumbFactory *factory;
 
     ThunarVfsPath         *path;
@@ -54,7 +56,7 @@ struct _RsttoNavigator
     GList                 *file_list;
     GList                 *file_iter;
     GList                 *history;
-    gint                   max_history;
+    gdouble                max_history;
 
     gboolean               preload;
 
@@ -95,7 +97,7 @@ gint
 rstto_navigator_get_old_position(RsttoNavigator *navigator);
 
 gint
-rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry);
+rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry, gboolean with_monitor);
 void
 rstto_navigator_remove (RsttoNavigator *navigator, RsttoNavigatorEntry *entry);
 gint
@@ -103,11 +105,9 @@ rstto_navigator_get_n_files (RsttoNavigator *navigator);
 RsttoNavigatorEntry *
 rstto_navigator_get_nth_file (RsttoNavigator *navigator, gint n);
 void
-rstto_navigator_set_file (RsttoNavigator *navigator, gint n);
+rstto_navigator_set_file_nr (RsttoNavigator *navigator, gint n);
 void
 rstto_navigator_clear (RsttoNavigator *navigator);
-void
-rstto_navigator_set_entry_rotation (RsttoNavigator *navigator, RsttoNavigatorEntry *entry, GdkPixbufRotation rotation);
 
 RsttoNavigatorEntry *
 rstto_navigator_entry_new (RsttoNavigator *, ThunarVfsInfo *info);
@@ -125,6 +125,8 @@ void
 rstto_navigator_entry_set_scale (RsttoNavigatorEntry *entry, gdouble);
 void
 rstto_navigator_entry_set_rotation (RsttoNavigatorEntry *entry, GdkPixbufRotation rotation);
+gint
+rstto_navigator_entry_get_position(RsttoNavigatorEntry *entry);
 
 GdkPixbufRotation
 rstto_navigator_entry_get_rotation (RsttoNavigatorEntry *entry);
@@ -133,7 +135,7 @@ rstto_navigator_entry_get_flip (RsttoNavigatorEntry *entry, gboolean horizontal)
 void
 rstto_navigator_flip_entry(RsttoNavigator *navigator, RsttoNavigatorEntry *entry, gboolean horizontal);
 GdkPixbuf *
-rstto_navigator_get_entry_thumb(RsttoNavigator *navigator, RsttoNavigatorEntry *entry, gint size);
+rstto_navigator_entry_get_thumb(RsttoNavigatorEntry *entry, gint size);
 ExifData *
 rstto_navigator_entry_get_exif_data (RsttoNavigatorEntry *entry);
 
@@ -143,9 +145,17 @@ GdkPixbuf *
 rstto_navigator_entry_get_pixbuf (RsttoNavigatorEntry *entry);
 gboolean
 rstto_navigator_entry_load_image (RsttoNavigatorEntry *entry, gboolean empty_cache);
+void
+rstto_navigator_entry_select (RsttoNavigatorEntry *entry);
 
-gint
-rstto_navigator_get_cache_max_images (RsttoNavigator *navigator);
+guint64
+rstto_navigator_entry_get_size (RsttoNavigatorEntry *entry);
+gdouble
+rstto_navigator_get_max_history_size(RsttoNavigator *navigator);
+void
+rstto_navigator_set_max_history_size(RsttoNavigator *nav, gdouble size);
+void
+rstto_navigator_set_monitor_handle_for_dir(RsttoNavigator *nav, ThunarVfsPath *dir_path);
 
 G_END_DECLS
 
